// RandomMotionNode.java
//
// Copyright (C) 1996 Dimension X, Inc.
//   Brett Levine 6-8-96

package dnx.lr.node;

import dnx.lr.*;
import dnx.lr.field.*;

/**
 * <PRE>
 *  RandomMotion {
 *     exposedField SFFloat      percent_jittery
 *     exposedField SFFloat      max_distance
 *     eventIn      SFFloat      set_fraction    
 *     eventOut     SFVec3f      value_changed        
 *  }
 * </PRE>
 * 
 *  <P>This node outputs a new random position relative to the old position
 *  This would be appropriate for randomizing a translation within the
params
 *  percent_jittery (how likely object is to move the maximum distance) and
 *  max_distance (the furthest distance away from its current position it
can
 *  move). </P>
 */

public class RandomMotionNode extends LeafNode
{
   public SFFloat percent_jittery;
   public SFFloat max_distance;
   public SFVec3f value_changed;

   private static int set_fractionIn;

   
   protected void createNodeDefinition(NodeDefinition def)
   {
      def.exposedField("percent_jittery", new SFFloat());
      def.exposedField("max_distance", new SFFloat());
      def.eventIn("set_fraction", new SFFloat());
      def.eventOut("value_changed", new SFVec3f());
      set_fractionIn = def.getEventInIndex("set_fraction");
   }
   
   protected void initFields()
   {
      percent_jittery = (SFFloat)getField("percent_jittery");
      max_distance = (SFFloat)getField("max_distance");
      value_changed = (SFVec3f)getEventOut("value_changed");
   }

   public void handleEvent(SceneEvent ev)
   {
      if (ev.getEventInIndex() == set_fractionIn) 
      {
         float scaleVal = percent_jittery.getValue() / 100f *
max_distance.getValue();
         value_changed.setValue((float)Math.random() * scaleVal,
                                (float)Math.random() * scaleVal,
                                (float)Math.random() * scaleVal);
      }
   }
}
